##
##  =============================================================================================
##  IBM Confidential
##  © Copyright IBM Corp. 2020
##  The source code for this program is not published or otherwise divested of its trade secrets,
##  irrespective of what has been deposited with the U.S. Copyright Office.
##  =============================================================================================
##


#!/bin/bash
# Must run as root
if [ `id -u` -ne 0 ]; then
   echo "Installation script can be executed only as root, Exiting.."
   exit 1
fi

# =================================
# Check for neccessary files
# =================================
# Check for config template file
CONFIG_TEMPLATE_FILE="${PWD}/telegraf.conf.template"
if ! [[ -f "${CONFIG_TEMPLATE_FILE}" ]]; then
    echo "${CONFIG_TEMPLATE_FILE} does not exist"
    exit 1
fi

# Check for sap-metrics-collector executable
BINARY_FILE="${PWD}/sap-metrics-collector"
if ! [[ -f "${BINARY_FILE}" ]]; then
    echo "${BINARY_FILE} does not exist"
    exit 1
fi

# Check for service template file
SERVICE_TEMPLATE_FILE="${PWD}/sap.service.template"
if ! [[ -f "${SERVICE_TEMPLATE_FILE}" ]]; then
    echo "${SERVICE_TEMPLATE_FILE} does not exist"
    exit 1
fi

# Check for sap.xml template file
XML_TEMPLATE_FILE="${PWD}/sap.xml"
if ! [[ -f "${XML_TEMPLATE_FILE}" ]]; then
    echo "${XML_TEMPLATE_FILE} does not exist"
    exit 1
fi

# Check for version file
VERSION_FILE="${PWD}/.version"
if ! [[ -f "${VERSION_FILE}" ]]; then
    echo "${VERSION_FILE} does not exist"
    exit 1
fi

# Check for auto_update file
UPDATE_FILE="${PWD}/auto_update.sh"
if ! [[ -f "${UPDATE_FILE}" ]]; then
    echo "${UPDATE_FILE} does not exist"
    exit 1
fi

# Check for logrotate file
LOGROTATE_FILE="${PWD}/sap_metric_logrotate"
SAP_LOGROTATE_FILE="sap_metric"
if ! [[ -f "${LOGROTATE_FILE}" ]]; then
    echo "${LOGROTATE_FILE} does not exist"
    exit 1
fi

# =================================
# Set up output directory
# =================================
OUTPUT_DIR='/etc/sap_metric_collector'
mkdir -p ${OUTPUT_DIR}
cp ${XML_TEMPLATE_FILE} ${OUTPUT_DIR}

# =================================
# Set up config file
# =================================
# Ask for API key
api_key=$1
if [ -z "$api_key" ]
then
	echo 'What is the API key received from ibm?'
	read api_key
fi

echo "Your API key is ${api_key}"

# Generate output config file by replace placeholder in config template file with supplied API key
OUTPUT_CONFIG_FILE="${OUTPUT_DIR}/telegraf.conf"
echo "Generating config file to ${OUTPUT_CONFIG_FILE}"
cp ${CONFIG_TEMPLATE_FILE} ${OUTPUT_CONFIG_FILE}
sed -i "s|<your BM api key here>|${api_key}|" ${OUTPUT_CONFIG_FILE}
sed -i "s|<your output metrics file path here>|${OUTPUT_DIR}/sap.new.xml|" ${OUTPUT_CONFIG_FILE}
sed -i "s|<your input metrics file path here>|${OUTPUT_DIR}/sap.xml|" ${OUTPUT_CONFIG_FILE}
sed -i "s|<your log file path here>|${OUTPUT_DIR}/sap.log|" ${OUTPUT_CONFIG_FILE}

echo "Generated config file"

# =================================
# Set up SAP linux service
# =================================
# Copy sap files to etc location
OUTPUT_BINARY_FILE="${OUTPUT_DIR}/sap-metrics-collector"
cp ${BINARY_FILE} ${OUTPUT_BINARY_FILE}

# Generate output service file by replacing placeholder in service template file 
OUTPUT_SERVICE_FILE="/etc/systemd/system/sap-metrics-collector.service"

echo "Generating service file to ${OUTPUT_SERVICE_FILE}"
cp ${SERVICE_TEMPLATE_FILE} ${OUTPUT_SERVICE_FILE}
USER="$(whoami)"
sed -i "s|<your username here>|${USER}|" ${OUTPUT_SERVICE_FILE}
sed -i "s|<your path to binary>|${OUTPUT_BINARY_FILE} --config=${OUTPUT_CONFIG_FILE}|" ${OUTPUT_SERVICE_FILE}

echo "Generated service file"

# Copy output service file to output directory for reference
cp ${OUTPUT_SERVICE_FILE} ${OUTPUT_DIR}

# Copy version file to output directory 
cp ${VERSION_FILE} ${OUTPUT_DIR}

# Copy version file to output directory 
cp ${UPDATE_FILE} ${OUTPUT_DIR}

# Copy lograte file to logrotate directory
LOGROTATE_DIR="/etc/logrotate.d"
if [ -d $LOGROTATE_DIR ]; then
    cp ${LOGROTATE_FILE} $LOGROTATE_DIR/${SAP_LOGROTATE_FILE}
fi

# Start sap-metric-collector
sudo systemctl start sap-metrics-collector
sudo systemctl enable sap-metrics-collector

# Starting auto-update
CRON_FILE=/var/spool/cron/root

if [ ! -f $CRON_FILE ]; then
   touch $CRON_FILE
   /usr/bin/crontab $CRON_FILE
fi


HOUR=`date +"%H"`
MINUTE=`date +"%M"`

/bin/echo "${MINUTE} ${HOUR} * * * bash -c 'cp ${OUTPUT_DIR}/auto_update.sh /tmp/cron_job.sh && /tmp/cron_job.sh' >/dev/null 2>&1" >> $CRON_FILE

# =================================
# Closing remarks
# =================================
echo "Installation Successful"
echo "--------------------------------------------------------------------------"
echo "Next steps:"
echo "1. Inspect files at ${OUTPUT_DIR}"
echo "2. Check if service running by 'sudo systemctl is-active sap-metrics-collector'"
echo "Expected successful running output: active" 
echo "--------------------------------------------------------------------------"
echo "Stopping the service:"
echo "1. Stop service by executing 'sudo systemctl stop sap-metrics-collector'"
echo "--------------------------------------------------------------------------"
